## Filename: UCS/R.pm
## Modified: Mon Mar 29 10:41:45 2004 (evert)   
##   Author: Stefan Evert
##  Purpose: interface to R  (communicates with R server process)

package UCS::R;

use UCS;
use UCS::Expression::Func;
use Expect;
use FileHandle;
use UCS::File;

$R = undef;					# Expect object for R server process
$R_Version = "";

# start R server process
sub Start ( ) {
  return 1 if defined $R;			# already running

  UCS::Die "Sorry, the R interpreter does not seem to be installed on your computer."
      unless defined $UCS::Rbin;

  UCS::Status "Spawning R server ... ";
  $R = Expect->spawn($UCS::Rbin, "--vanilla", "--no-readline"); # R must be in standard search path
  UCS::Die "Couldn't start R process in background.\n"
      unless defined $R;
  $R->log_stdout(0);

  my (undef, $error) =				# wait for version information
    $R->expect(15,
	       ["-re", 'Version\s+([0-9]+(\.[0-9]+)+)',
		sub { my $obj = shift; 
		      $R_Version = ($obj->exp_matchlist)[0];
		      exp_continue;
		    }
	       ],
	       "-re", '^>');
  UCS::Die "Couldn't start R process: $error\n"
    if defined $error;

  $R->clear_accum;
  print $R "options(digits=22, width=10)\n";
  (undef, $error) = $R->expect(5, "-re", '^>');
  UCS::Die "Couldn't setup R process: $error\n"
      if defined $error;
  print $R "source('$UCS::R_BaseDir/R/lib/ucs.R'); ucs.library(\"sfunc\")\n";
  (undef, $error) = $R->expect(5, "-re", '^>');
  UCS::Die "Couldn't load R libraries: $error\n"
      if defined $error;
  
  UCS::Status "R Version $R_Version running.\n";
  return 1;
}

END {
  # make sure to stop the server when the program exits or dies
  # (otherwise Expect will hang up when the object $R ist destroyed)
  Stop() if defined $R;
  # perhaps it's safer to store the server's PID and kill it here? ("kill 9, $pid;")
}

# stop R server process
sub Stop ( ) {
  return 1 unless defined $R;			# no server running
  
  UCS::Status "Stopping R server ... ";
  print $R "q()\n";
  # should check whether R has really exited and kill it if necessary, otherwise the next line will hang
  undef($R);
  $R_Version = "";

  UCS::Status "ok\n";
  return 1;
}

# @results = UCS::R::Exec("$cmd");
# execute R command and read result (which must be a numeric vector)
sub Exec ( $ ) {
  my $cmd = shift;
  if (defined $R and defined $R->exp_exitstatus) { # re-start R server if it has died or something
    undef $R;
  }
  Start() unless defined $R;			# auto-start R server when necessary
  $cmd =~ tr[\n][ ]s;				# command should be on single line
  print $R "$cmd\n";
  my @Result = ();
  $R->expect(60,
	     ["-re", qr/^\s*\[([0-9]+)\]\s*(.*?)\r?\n/,
	      sub { my $obj = shift; 
		    my ($idx, $line) = $obj->exp_matchlist;
		    my @items = split " ", $line;
		    while (@items) {
		      my $exp_idx = @Result + 1;
		      UCS::Die "UCS::R: indexing problem with R output (expected $exp_idx, got $idx)"
			  unless $exp_idx == $idx;
		      push @Result, shift @items;
		      $idx++;
		    }
		    exp_continue;
		  }
	     ],
	     "-re", "^>");
  UCS::Warn "UCS::R: communication error with R backend (".$R->error.")"
    if $R->error;
  return (wantarray) ? @Result : $Result[0];
}

# UCS::R::LoadVector($varname, \@data);
# load numeric vector into R through temporary file, using the scan() function
sub LoadVector ( $$ ) {
  my $var = shift;
  my $data = shift;
  my $l = scalar @$data;
  if ($l == 0) {
    Exec("$var <- numeric(0)");
  }
  else {
    my $tf = new UCS::File::Temp;
    for my $i (0 .. $l - 1) {
      $tf->write($data->[$i], "\n");
    }
    $tf->finish;
    my $filename = $tf->name;
    Exec("$var <- scan(file=\"$filename\", quiet=TRUE, sep=\"\\n\", n=$l)");
    $tf->close;
  }
  my $l2 = Exec("length($var)");
  UCS::Die "UCS::R::LoadVector(): load failed (sent $l entries, received $l2)"
      unless $l2 == $l;
}

# $arrayref = UCS::R::DumpVector($varname);
# read numeric vector from R through temporary file, using the write() function;
# the data are returned as an anonymous array reference for efficiency reasons
sub DumpVector ( $ ) {
  my $var = shift;
  my @data = ();
  my $l = Exec("length($var)");
  if ($l == 0) {
    return [];
  }
  else {
    my $tf = new UCS::File::Temp;
    $tf->finish;
    my $filename = $tf->name;
    Exec("write($var, file=\"$filename\", ncolumns=1)");
    $data[$l-1] = 0;				# pre-allocate array
    my $size = 0;
    while (my $line = $tf->read) {
      chomp $line;
      $data[$size] = $line;
      $size++;
    }
    $tf->close;
    UCS::Die "UCS::R::DumpVector(): dump failed (expected $l entries, got $size)"
	unless $size == $l;
    return \@data;
  }
}

##
## SPECIAL FUNCTIONS  (see documentation in UCS::SFunc package)
##

# $coef = choose($n, $k);
# $log_coef = lchoose($n, $k);
# interface to R functions choose() and lchoose() for binomial coefficient and its logarithm
sub choose ( $$ ) {
  my ($n, $k) = @_;
  return Exec("choose($n, $k)");
}
sub lchoose ( $$ ) {
  my ($n, $k) = @_;
  return Exec("lchoose($n, $k) / log(10)");
}

# $g = gamma($a);
# $log_g = lgamma($a);
# interface to R functions gamma() and lgamma() for Gamma function and its logarithm
sub gamma ( $ ) {
  my $a = shift;
  return Exec("Cgamma($a)");
}
sub lgamma ( $ ) {
  my $a = shift;
  return Exec("Cgamma($a, log=TRUE)");
}

# $g = igamma($a, $x [, $upper]);
# $rg = rgamma($a, $x [, $upper]);
# $log_g = ligamma($a, $x [, $upper]);
# $log_rg = lrgamma($a, $x [, $upper]);
# incomplete and regularised Gamma functions (lower and upper), plus their logarithms
sub igamma ( $$;$ ) {
  my ($a, $x, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Igamma($a, $x, $tail)");
}
sub rgamma ( $$;$ ) {
  my ($a, $x, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Rgamma($a, $x, $tail)");
}
sub ligamma ( $$;$ ) {
  my ($a, $x, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Igamma($a, $x, $tail, log=TRUE)");
}
sub lrgamma ( $$;$ ) {
  my ($a, $x, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Rgamma($a, $x, $tail, log=TRUE)");
}

# $x = igamma_inv($a, $g [, $upper]);
# $x = rgamma_inv($a, $rg [, $upper]);
# $x = ligamma_inv($a, $log_g [, $upper]);
# $x = lrgamma_inv($a, $log_rg [, $upper]);
# inverse of incomplete and regularised Gamma functions and their logarithms
sub igamma_inv ( $$;$ ) {
  my ($a, $y, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Igamma.inv($a, $y, $tail)");
}
sub rgamma_inv ( $$;$ ) {
  my ($a, $y, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Rgamma.inv($a, $y, $tail)");
}
sub ligamma_inv ( $$;$ ) {
  my ($a, $y, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Igamma.inv($a, $y, $tail, log=TRUE)");
}
sub lrgamma_inv ( $$;$ ) {
  my ($a, $y, $upper) = @_;
  my $tail = ($upper) ? "lower=FALSE" : "lower=TRUE";
  return Exec("Rgamma.inv($a, $y, $tail, log=TRUE)");
}

# $b = beta($a, $b);
# $log_b = lbeta($a, $b);
# interface to R functions beta() and lbeta() for Beta function and its logarithm
sub beta ( $ ) {
  my ($a, $b) = @_;
  return Exec("Cbeta($a, $b)");
}sub lbeta ( $ ) {
  my ($a, $b) = @_;
  return Exec("Cbeta($a, $b, log=TRUE)");
}

# $b = ibeta($x, $a, $b);
# $br = rbeta($x, $a, $b);
# $log_b = libeta($x, $a, $b);
# $log_br = lrbeta($x, $a, $b);
# incomplete and regularised Beta functions and their logarithms
sub ibeta ( $$$ ) {
  my ($x, $a, $b) = @_;
  return Exec("Ibeta($x, $a, $b)");
}
sub rbeta ( $$$ ) {
  my ($x, $a, $b) = @_;
  return Exec("Rbeta($x, $a, $b)");
}
sub libeta ( $$$ ) {
  my ($x, $a, $b) = @_;
  return Exec("Ibeta($x, $a, $b, log=TRUE)");
}
sub lrbeta ( $$$ ) {
  my ($x, $a, $b) = @_;
  return Exec("Rbeta($x, $a, $b, log=TRUE)");
}

# $x = ibeta_inv($b, $a, $b);
# $x = rbeta_inv($br, $a, $b);
# $x = libeta_inv($log_b, $a, $b);
# $x = lrbeta_inv($log_br, $a, $b);
# inverse of incomplete and regularised Beta functions and their logarithms
sub ibeta_inv ( $$$ ) {
  my ($y, $a, $b) = @_;
  return Exec("Ibeta.inv($y, $a, $b)");
}
sub rbeta_inv ( $$$ ) {
  my ($y, $a, $b) = @_;
  return Exec("Rbeta.inv($y, $a, $b)");
}
sub libeta_inv ( $$$ ) {
  my ($y, $a, $b) = @_;
  return Exec("Ibeta.inv($y, $a, $b, log=TRUE)");
}
sub lrbeta_inv ( $$$ ) {
  my ($y, $a, $b) = @_;
  return Exec("Rbeta.inv($y, $a, $b, log=TRUE)");
}


##
## STATISTICAL DISTRIBUTIONS  (see documentation in UCS::SFunc package)
##

# $d = dbinom($k, $size, $prob);
# $ld = ldbinom($k, $size, $prob);
# $p = pbinom($k, $size, $prob [, $upper]);
# $lp = lpbinom($k, $size, $prob [, $upper]);
# $k = qbinom($p, $size, $prob [, $upper]);
# $k = lqbinom($lp, $size, $prob [, $upper]);
# density, tail probabilities, and quantiles of binomial distribution (incl. logarithmic versions)
sub dbinom ( $$$ ) {
  my ($k, $size, $prob) = @_;
  return Exec("dbinom($k, $size, $prob)");
}
sub ldbinom ( $$$ ) {
  my ($k, $size, $prob) = @_;
  return Exec("-dbinom($k, $size, $prob, log=TRUE) / log(10)");
}
## R implementation of pbinom() is slow and unreliable (logarithmic version doesn't prevent underflow);
## these are just for reference
# sub pbinom_R ( $$$;$ ) {
#   my ($k, $size, $prob, $upper) = @_;
#   my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
#   return Exec("pbinom($k, $size, $prob, $tail)");
# }
# sub lpbinom_R ( $$$;$ ) {
#   my ($k, $size, $prob, $upper) = @_;
#   my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
#   return Exec("-pbinom($k, $size, $prob, $tail, log=TRUE) / log(10)");
# }
sub pbinom ( $$$;$ ) {
  my ($k, $size, $prob, $upper) = @_;
  if ($upper and $size >= 1e6 and $prob > 0 and $prob < 0.5) { # use multiplication algorithm for large N and small p
    # calculate sum(j=k+1:N, dbinom(j, size, prob)) using the ratio between successive terms
    #   r(j) = dbinom(j+1, size, prob) / dbinom(j, size, prob) = (N - k) / (k + 1) * p / (1 - p)
    my $odds = $prob / (1 - $prob);
    my $k_bd = $size / (0.7 / $odds + 1); # r(j) <= 0.7 for j >= k_bd (-> convergence within at most 100 steps)
    if ($k_bd - $k <= 100) {			# this ensures that we reach convergence within at most 200 steps
      my $p = Exec("dbinom(".($k+1).", $size, $prob)");
      my $cum_p = $p;
      my $last_p = $cum_p;
      for (my $j = $k + 1; $j < $size; $j++) {   
	$p *= ($size - $k) / ($k + 1) * $odds;	# = dbinom(j+1, size, prob)
	$cum_p += $p;				# add to cumulative probability
	last if $cum_p == $last_p;		# "empirical"convergence
	$last_p = $cum_p;
      }
      return $cum_p;
    }
  }
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("pbinom($k, $size, $prob, $tail)"); # use R function if not handled by multiplication algorithm
}
sub lpbinom ( $$$;$ ) {
  my ($k, $size, $prob, $upper) = @_;
  if ($upper and $size >= 1e6 and $prob > 0 and $prob < 0.5) { # use multiplication algorithm for large N and small p
    # same as above, but compute dbinom(k+1, size, prob) and the factor 1 + r(k+1) + r(k+1)*r(k+2) ...
    # separately, and add their logarithms (to avoid floating-point underflow)
    my $odds = $prob / (1 - $prob);
    my $k_bd = $size / (0.7 / $odds + 1); # r(j) <= 0.7 for j >= k_bd (-> convergence within at most 100 steps)
    if ($k_bd - $k <= 100) {			# this ensures that we reach convergence within at most 200 steps
      my $term = 1;
      my $cum_factor = $term;
      my $last_factor = $cum_factor;
      for (my $j = $k + 1; $j < $size; $j++) {   
	$term *= ($size - $k) / ($k + 1) * $odds; # = dbinom(j+1, size, prob) / dbinom(k+1, size, prob)
	$cum_factor += $term;				# add to factor (cumulative probability / P(k+1))
	last if $cum_factor == $last_factor;		# "empirical"convergence
	$last_factor = $cum_factor;
      }
      return ldbinom($k+1, $size, $prob) - log10($cum_factor); # = -log10(dbinom(k+1, ...) * factor)
    }
  }
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("-pbinom($k, $size, $prob, $tail, log=TRUE) / log(10)");
}
sub qbinom ( $$$;$ ) {
  my ($p, $size, $prob, $upper) = @_;
  my $tail = ($upper) ? ", lower.tail=FALSE" : "";
  return Exec("qbinom($p, $size, $prob $tail)");
}
sub lqbinom ( $$$;$ ) {
  my ($lp, $size, $prob, $upper) = @_;
  my $tail = ($upper) ? ", lower.tail=FALSE" : "";
  return Exec("qbinom(-$lp * log(10), $size, $prob $tail, log=TRUE)");
}

# $d = dpois($k, $lambda);
# $ld = ldpois($k, $lambda);
# $p = ppois($k, $lambda [, $upper]);
# $lp = lppois($k, $lambda [, $upper]);
# $k = qpois($p, $lambda [, $upper]);
# $k = lqpois($lp, $lambda [, $upper]);
# density, tail probabilities, and quantiles of Poisson distribution (incl. logarithmic versions)
sub dpois ( $$ ) {
  my ($k, $lambda) = @_;
  return Exec("dpois($k, $lambda)");
}
sub ldpois ( $$ ) {
  my ($k, $lambda) = @_;
  return Exec("-dpois($k, $lambda, log=TRUE) / log(10)");
}
sub ppois ( $$;$ ) {
  my ($k, $lambda, $upper) = @_;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("ppois($k, $lambda, $tail)");
}
sub lppois ( $$;$ ) {
  my ($k, $lambda, $upper) = @_;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("-ppois($k, $lambda, $tail, log=TRUE) / log(10)");
}
sub qpois ( $$;$ ) {
  my ($p, $lambda, $upper) = @_;
  my $tail = ($upper) ? ", lower.tail=FALSE" : "";
  return Exec("qpois($p, $lambda $tail)");
}
sub lqpois ( $$;$ ) {
  my ($lp, $lambda, $upper) = @_;
  my $tail = ($upper) ? ", lower.tail=FALSE" : "";
  return Exec("qpois(-$lp * log(10), $lambda $tail, log=TRUE)");
}

# $d = dnorm($x, $mu, $sigma);
# $ld = ldnorm($x, $mu, $sigma);
# $p = pnorm($x, $mu, $sigma [, $upper]);
# $lp = lpnorm($x, $mu, $sigma [, $upper]);
# $x = qnorm($p, $mu, $sigma [, $upper]);
# $x = lqnorm($lp, $mu, $sigma [, $upper]);
# density, tail probabilities, and quantiles of normal distribution (incl. logarithmic versions)
sub dnorm ( $;$$ ) {
  my $x = shift;
  my $mu = (@_) ? shift : 0;
  my $sigma = (@_) ? shift : 1;
  return Exec("dnorm($x, mean=$mu, sd=$sigma)");
}
sub ldnorm ( $;$$ ) {
  my $x = shift;
  my $mu = (@_) ? shift : 0;
  my $sigma = (@_) ? shift : 1;
  return Exec("-dnorm($x, mean=$mu, sd=$sigma, log=TRUE) / log(10)");
}
sub pnorm ( $;$$$ ) {
  my $x = shift;
  my $mu = (@_) ? shift : 0;
  my $sigma = (@_) ? shift : 1;
  my $upper = (@_) ? shift : 0;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("pnorm($x, mean=$mu, sd=$sigma, $tail)");
}
sub lpnorm ( $;$$$ ) {
  my $x = shift;
  my $mu = (@_) ? shift : 0;
  my $sigma = (@_) ? shift : 1;
  my $upper = (@_) ? shift : 0;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("-pnorm($x, mean=$mu, sd=$sigma, $tail, log=TRUE) / log(10)");
}
sub qnorm ( $;$$$ ) {
  my $x = shift;
  my $mu = (@_) ? shift : 0;
  my $sigma = (@_) ? shift : 1;
  my $upper = (@_) ? shift : 0;
  my $tail = ($upper) ? ", lower.tail=FALSE" : "";
  return Exec("qnorm($p, $mu, $sigma $tail)");
}
sub lqnorm ( $;$$$ ) {
  my $x = shift;
  my $mu = (@_) ? shift : 0;
  my $sigma = (@_) ? shift : 1;
  my $upper = (@_) ? shift : 0;
  my $tail = ($upper) ? ", lower.tail=FALSE" : "";
  return Exec("qnorm(-$lp * log(10), $mu, $sigma $tail, log=TRUE)");
}

# $d = dchisq($x, $df);
# $ld = ldchisq($x, $df);
# $p = pchisq($x, $df [, $upper]);
# $lp = lpchisq($x, $df [, $upper]);
# $x = qchisq($p, $df [, $upper]);
# $x = lqchisq($lp, $df [, $upper]);
# density, tail probabilities, and quantiles of chi-squared distribution
sub dchisq ( $$ ) {
  my ($x, $df) = @_;
  return Exec("dchisq($x, $df)");
}
sub ldchisq ( $$ ) {
  my ($x, $df) = @_;
  return Exec("-dchisq($x, $df, log=TRUE) / log(10)");
}
sub pchisq ( $$;$ ) {
  my ($x, $df, $upper) = @_;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("pchisq($x, $df, $tail)");
}
sub lpchisq ( $$;$ ) {
  my ($x, $df, $upper) = @_;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("-pchisq($x, $df, $tail, log=TRUE) / log(10)");
}
sub qchisq ( $$;$ ) {
  my ($p, $df, $upper) = @_;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("qchisq($p, $df, $tail)");
}
sub lqchisq ( $$;$ ) {
  my ($lp, $df, $upper) = @_;
  my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
  return Exec("qchisq(-$lp * log(10), $df, $tail)");
}

# $d = dhyper($k, $R1, $R2, $C1, $C2);
# $ld = ldhyper($k, $R1, $R2, $C1, $C2);
# $p = phyper($k, $R1, $R2, $C1, $C2 [, $upper]);
# $lp = lphyper($k, $R1, $R2, $C1, $C2 [, $upper]);
# density and tail probabilities of hypergeometric distribution, using a mixed implementation for improved accuracy
sub dhyper ( $$$$$ ) {
  my ($k, $R1, $R2, $C1, $C2) = @_;
  my $N = $R1 + $R2;
  UCS::Die "UCS::R:  sum of rows $R1 + $R2 != sum of columns $C1 + $C2 in dhyper() function"
      unless $N == $C1 + $C2;
  my $min_k = max(0, $R1 - $C2);
  my $max_k = min($R1, $C1);
  return 0 
    if $k < $min_k or $k > $max_k;
  return Exec("dhyper($k, $C1, $C2, $R1)");
}
sub ldhyper ( $$$$$ ) {
  my ($k, $R1, $R2, $C1, $C2) = @_;
  my $N = $R1 + $R2;
  UCS::Die "UCS::R:  sum of rows $R1 + $R2 != sum of columns $C1 + $C2 in ldhyper() function"
      unless $N == $C1 + $C2;
  my $min_k = max(0, $R1 - $C2);
  my $max_k = min($R1, $C1);
  return inf()
    if $k < $min_k or $k > $max_k;
  return Exec("-dhyper($k, $C1, $C2, $R1, log=TRUE) / log(10)");
}
sub phyper ( $$$$$;$ ) {
  my ($k, $R1, $R2, $C1, $C2, $upper) = @_;
  my $N = $R1 + $R2;
  UCS::Die "UCS::R:  sum of rows $R1 + $R2 != sum of columns $C1 + $C2 in phyper() function"
      unless $N == $C1 + $C2;
  my $min_k = max(0, $R1 - $C2);
  my $max_k = min($R1, $C1);
  if ($k < $min_k) {
    return ($upper) ? 1 : 0;
  }
  elsif ($k >= $max_k) {
    return ($upper) ? 0 : 1;
  }
  elsif ($upper and $k*$N > $R1*$C1 ) {		# if $k >= E[k], use multiplication algorithm for upper tail
    # we have to calculate sum(j=k+1:max_k, dhyper(j, R1, R2, C1, C2)); using the ratio between successive terms:
    #   r(j+1) = dhyper(j+1, ...) / dhyper(j, ...) = (C1-j)*(R1-j) / ((j+1)*(C2-R1+j+1))
    # this sum is computed efficiently; note that r(j) is decreasing and r(j) < 1 for j > E[k] so we
    # always have geometric convergence and can sum until $max_k is reached or the sum converges
    my $p = dhyper($k+1, $R1, $R2, $C1, $C2);
    my $cum_p = $p;
    my $last_p = $cum_p;
    for (my $j = $k+1; $j < $max_k; $j++) {
      my $r = ($C1 - $j) * ($R1 - $j) / (($j + 1) * ($C2 - $R1 + $j + 1)); # = r(j+1)
      $p *= $r;					# = dhyper(j+1, ...)
      $cum_p += $p;				# add to cumulative probability
      last if $cum_p == $last_p;		# "empirical" convergence
      $last_p = $cum_p;
    }
    return $cum_p;
  }
  else {					# use R for lower tail and for upper tail when p >= 0.5
    my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
    return Exec("phyper($k, $C1, $C2, $R1, $tail)");
  }
}
sub lphyper ( $$$$$;$ ) {
  my ($k, $R1, $R2, $C1, $C2, $upper) = @_;
  my $N = $R1 + $R2;
  UCS::Die "UCS::R:  sum of rows $R1 + $R2 != sum of columns $C1 + $C2 in phyper() function"
      unless $N == $C1 + $C2;
  my $min_k = max(0, $R1 - $C2);
  my $max_k = min($R1, $C1);
  if ($k < $min_k) {
    return ($upper) ? 0 : inf();
  }
  elsif ($k >= $max_k) {
    return ($upper) ? inf() : 0;
  }
  elsif ($upper and $k*$N > $R1*$C1 ) {		# if $k >= E[k], use multiplication algorithm for upper tail
    # same as above, but compute dhyper(k+1, ...) and the factor 1 + r(k+2) + r(k+2)*r(k+3) + ... 
    # separately and add the logarithms (to avoid floating-point underflow problems)
    my $term = 1;
    my $cum_factor = 1;
    my $last_factor = $cum_factor;
    for (my $j = $k+1; $j < $max_k; $j++) {
      my $r = ($C1 - $j) * ($R1 - $j) / (($j + 1) * ($C2 - $R1 + $j + 1)); # = r(j+1)
      $term *= $r;				# = r(k+2) * ... * r(j+1)
      $cum_factor += $term;
      last if $cum_factor == $last_factor;	# "empirical" convergence
      $last_factor = $cum_factor;
    }
    return ldhyper($k+1, $R1, $R2, $C1, $C2) - log10($cum_factor); # = -log10(dhyper(k+1, ...) * factor)
  }
  else {					# use R for lower tail and for upper tail when p >= 0.5
    my $tail = ($upper) ? "lower.tail=FALSE" : "lower.tail=TRUE";
    return Exec("-phyper($k, $C1, $C2, $R1, $tail, log=TRUE) / log(10)");
  }
}


1;



__END__


=head1 NAME

UCS::R - UCS/Perl interface to R

=head1 SYNOPSIS

  use UCS::R;

  UCS::R::Start();		 # start R backend explicitly
  UCS::R::Stop();                # terminate R backend

  @x = UCS::R::Exec($cmd);	 # execute R cmd (must return numeric vector)

  UCS::R::LoadVector("my.x", \@data);  # load numeric vector efficiently into R
  $data = UCS::R::DumpVector("my.x");  # returns arrayref

  # access to special functions and statistical distributions
  # through the UCS::SFunc module

=head1 DESCRIPTION

The B<UCS::R> module provides an interface to the B<UCS/R> part of the UCS
system, managing communication with an B<R> server process running in the
background.  The R server automatically loads the B<UCS/R> base library and
the B<sfunc> extension module.

This interface is mainly used by the B<UCS::SFunc> module to make the R
implementations of B<special functions> (binomial coefficients, Gamma
function, Beta function) and B<statistical distributions> (binomial, Poisson,
normal, chi-squared, hypergeometric) available to B<UCS/Perl>, without relying
on an external maths library and/or compiled C code.

=head1 FUNCTIONS

=over 4

=item UCS::R::Start();

Starts B<R> server process.  Normally, this function does not have to be
called explicitly, as the server is automatically launched when an R command
is executed for the first time.  Since this will block program execution for a
few seconds, some scripts may prefer to call B<UCS::R::Start> at start-up
time before the R process is actually needed.

=item UCS::R::Stop();

Terminate the B<R> server process.  Normally, this function does not have to
be called explicitly, but it may be used to shut down an R process that is no
longer needed and free memory resources.

=item @x = UCS::R::Exec($cmd);

Executes the B<R> command I<$cmd> in the server process.  The command must
return a numeric vector, which is passed back to the calling script in the
form of a list I<@x>.  When command execution fails or its return value cannot
be parsed, the B<UCS::R::Exec> function will B<die> with an error message.

B<NB:> This interface is not efficient for exchanging large amounts of data
with R and may hang if the input/output buffers overflow.  Use the
B<LoadVector> and B<DumpVector> functions for this purpose (see below).
Moreover, I<$cmd> must be a single-line command (separate multiple commands
with C<;>), so that it leaves a single command prompt at the beginning of a
line after execution.  Avoid C<cat()> and any functions that prompt for user
input, otherwise B<UCS::R::Exec> will become confused and may hang.

=item UCS::R::LoadVector($varname, \@data);

Efficiently loads a numeric vector into B<R> (making use of a temporary file
and the C<scan()> function).  The data I<@data> is passed in as an array
reference and will be stored in the B<R> variable I<$varname>.

=item $data = UCS::R::DumpVector($varname);

Efficiently reads a numeric vector from B<R> (making use of a temporary file
and the C<write()> function).  The data stored in the B<R> variable
I<$varname> (which must be a numeric vector) is returned as an anonymous array
reference I<$data>.

=back


=head1 SPECIAL FUNCTIONS AND STATISTICAL DISTRIBUTIONS

The special functions and statistical distributions provided through the B<R>
interface are not exported by this module.  Use B<UCS::SFunc> instead.  All
available functions are documented in L<the UCS::SFunc manpage|UCS::SFunc>.
They are available under the same names in the B<UCS::R> package.  For
instance, the R implementation of the B<lgamma> function can be accessed 
explicitly as B<UCS::R::lgamma>. 


=head1 COPYRIGHT

Copyright 2004 Stefan Evert.

This software is provided AS IS and the author makes no warranty as to
its use and performance. You may use the software, redistribute and
modify it under the same terms as Perl itself.

=cut

